﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace UseUwpApiWPF2
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        private Windows.UI.Input.Inking.InkStrokeContainer _inkStrokeContainer;

        public MainWindow()
        {
            InitializeComponent();
            this._inkStrokeContainer = new Windows.UI.Input.Inking.InkStrokeContainer();
            this.WPFInkCanvas.EditingMode = InkCanvasEditingMode.None;
        }

        private void LoadToInkCanvasButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            Microsoft.Win32.OpenFileDialog dlgOpen = new Microsoft.Win32.OpenFileDialog();
            dlgOpen.Filter = "ISF(*isf,*.isfmss)|*.isf;*.isfmss";
            if ((bool)dlgOpen.ShowDialog())
            {
                // 現在のストロークをクリア
                this.WPFInkCanvas.Strokes.Clear();
                using (System.IO.FileStream fs =
                    new System.IO.FileStream(dlgOpen.FileName, System.IO.FileMode.Open))
                {
                    this.WPFInkCanvas.Strokes = new System.Windows.Ink.StrokeCollection(fs);
                }
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void CopyToClipboardFromInkCanvasButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var strokeCollection = this.WPFInkCanvas.Strokes;
            if (strokeCollection.Count > 0)
            {
                InkCanvasEditingMode icem = this.WPFInkCanvas.EditingMode;

                this.WPFInkCanvas.Select(strokeCollection);
                this.WPFInkCanvas.CopySelection();

                //this.InkCanvas1.EditingMode = icem;
            }


            this.MyProgressBar.IsIndeterminate = false;
        }

        private void PasteFromClipboardButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            this._inkStrokeContainer.Clear();
            if (this._inkStrokeContainer.CanPasteFromClipboard())
            {
                try
                {
                    //var uwpinkcanvas = new Windows.UI.Xaml.Controls.InkCanvas();
                    //var inkpresenter = new Windows.UI.Input.Inking.InkPresenter
                    var p = new Windows.Foundation.Point(0, 0);

                    ////////var rect = this._inkStrokeContainer.PasteFromClipboard(p); //ここで止まる。

                    var inkManager = new Windows.UI.Input.Inking.InkManager();
                    if(inkManager.CanPasteFromClipboard())
                    {
                        inkManager.PasteFromClipboard(p); // ここで止まる。
                    }
                    this._inkStrokeContainer.AddStrokes(inkManager.GetStrokes());

                    var strokes = this._inkStrokeContainer.GetStrokes();
                    var stroke = strokes[strokes.Count - 1];
                    this.PasteFromClipboardTextBox.Text = stroke.GetInkPoints()[0].Position.X.ToString() + " "
                        + stroke.GetInkPoints()[0].Position.Y.ToString() + " "
                        + stroke.DrawingAttributes.Size.Width.ToString() + " "
                        + stroke.DrawingAttributes.Size.Height.ToString() + " last stroke's information";
                }
                catch
                { }

            }
            else
            {
                // Cannot paste from clipboard.
            }


            //var sc = new StrokeCollection();
            //sc.Add(this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1]);
            //this.WPFInkCanvas.Select(sc);

            this.MyProgressBar.IsIndeterminate = false;
        }

        private async void SaveAsGIFButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            // Get all strokes on the InkCanvas.
            IReadOnlyList<Windows.UI.Input.Inking.InkStroke> currentStrokes = this._inkStrokeContainer.GetStrokes();

            // Strokes present on ink canvas.
            if (currentStrokes.Count > 0)
            {
                Microsoft.Win32.SaveFileDialog dlgOpen = new Microsoft.Win32.SaveFileDialog();
                dlgOpen.Filter = "GIF with embedded ISF(*gif,*.gifmss)|*.gif;*.gifmss";


                // When chosen, picker returns a reference to the selected file.
                if ((bool)dlgOpen.ShowDialog())
                {
                    Windows.Storage.StorageFolder folder = await Windows.Storage.StorageFolder.GetFolderFromPathAsync(System.IO.Path.GetDirectoryName(dlgOpen.FileName));
                    Windows.Storage.StorageFile file = await folder.CreateFileAsync(System.IO.Path.GetFileName(dlgOpen.FileName), Windows.Storage.CreationCollisionOption.ReplaceExisting);
                    //await Windows.Storage.StorageFile.GetFileFromPathAsync(dlgOpen.FileName);

                    // Prevent updates to the file until updates are 
                    // finalized with call to CompleteUpdatesAsync.
                    Windows.Storage.CachedFileManager.DeferUpdates(file);

                    // Open a file stream for writing.
                    Windows.Storage.Streams.IRandomAccessStream stream = await file.OpenAsync(Windows.Storage.FileAccessMode.ReadWrite);
                    // Write the ink strokes to the output stream.
                    using (Windows.Storage.Streams.IOutputStream outputStream = stream.GetOutputStreamAt(0))
                    {
                        await this._inkStrokeContainer.SaveAsync(outputStream);
                        await outputStream.FlushAsync();
                    }
                    stream.Dispose();

                    try
                    {
                        // Finalize write so other apps can update file.
                        Windows.Storage.Provider.FileUpdateStatus status =
                            await Windows.Storage.CachedFileManager.CompleteUpdatesAsync(file);

                        if (status == Windows.Storage.Provider.FileUpdateStatus.Complete)
                        {
                            // File saved.
                        }
                        else
                        {
                            // File couldn&#39;t be saved.
                        }
                    }
                    catch (Exception e3)
                    {
                        MessageBox.Show(e3.ToString());
                    }

                }
                // User selects Cancel.
                else
                {
                    // Operation cancelled.
                }
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void MarkerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            double d = 10;
            double penSize = d * 2;

            StylusPoint pointOne;
            StylusPointCollection points;
            DrawingAttributes da;
            Stroke stroke;

            points = new StylusPointCollection();
            pointOne = new StylusPoint(d, d, 0.5f);
            points.Add(pointOne);
            da = this.WPFInkCanvas.DefaultDrawingAttributes.Clone();
            da.Width = penSize;
            da.Height = penSize;
            da.StylusTip = StylusTip.Rectangle;
            stroke = new Stroke(points, da);
            this.WPFInkCanvas.Strokes.Add(stroke);

            this.MarkerTextBox.Text = stroke.GetBezierStylusPoints()[0].X.ToString() + " "
                + stroke.GetBezierStylusPoints()[0].Y.ToString() + " "
                + stroke.DrawingAttributes.Width + " "
                + stroke.DrawingAttributes.Height + " "
                + "Marker's information "
                + stroke.StylusPoints[0].X.ToString() + " " + stroke.StylusPoints[0].Y.ToString();

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void AnalyzeMarkerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var uwp_strokes = this._inkStrokeContainer.GetStrokes();
            if (uwp_strokes.Count > 0)
            {
                var stroke = uwp_strokes[uwp_strokes.Count - 1];
                var points = stroke.GetInkPoints();
                double startx = stroke.BoundingRect.X + stroke.BoundingRect.Width / 2; // points[0].Position.X;
                double starty = stroke.BoundingRect.Y + stroke.BoundingRect.Height / 2; // points[0].Position.Y;
                double penwidth = stroke.BoundingRect.Width; // stroke.DrawingAttributes.Size.Width;
                double penheight = stroke.BoundingRect.Height; // stroke.DrawingAttributes.Size.Height;

                foreach (var s in uwp_strokes)
                {
                    s.Selected = true;
                }
                this._inkStrokeContainer.CopySelectedToClipboard();

                this._inkStrokeContainer.Clear();
                if (this._inkStrokeContainer.CanPasteFromClipboard())
                {
                    this._inkStrokeContainer.PasteFromClipboard(new Windows.Foundation.Point(penwidth / 2 - startx, penheight / 2 - starty));
                }

                stroke = this._inkStrokeContainer.GetStrokes()[this._inkStrokeContainer.GetStrokes().Count - 1];
                this.AnalyzeMarkerButtonTextBox.Text = (stroke.BoundingRect.X + stroke.BoundingRect.Width / 2).ToString() + " "
                    + (stroke.BoundingRect.Y + stroke.BoundingRect.Height / 2).ToString() + " "
                    + stroke.BoundingRect.Width.ToString() + " "
                    + stroke.BoundingRect.Height.ToString()
                    + " last stroke's information";

                var sc = new StrokeCollection();
                sc.Add(this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1]);
                this.WPFInkCanvas.Select(sc);
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void ConvertStrokesButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var wpf_strokes = this.WPFInkCanvas.Strokes;
            if(wpf_strokes.Count > 0)
            {
                this._inkStrokeContainer.Clear();
                foreach(Stroke s in wpf_strokes)
                {
                    var points = s.GetBezierStylusPoints();
                    List<Windows.UI.Input.Inking.InkPoint> uwp_points = new List<Windows.UI.Input.Inking.InkPoint>();
                    foreach(Point p in points)
                    {
                        uwp_points.Add(new Windows.UI.Input.Inking.InkPoint(new Windows.Foundation.Point(p.X, p.Y), 0.5f));
                    }
                    Windows.UI.Input.Inking.InkStrokeBuilder builder = new Windows.UI.Input.Inking.InkStrokeBuilder();
                    var uwp_s = builder.CreateStrokeFromInkPoints(uwp_points, Matrix3x2.Identity);
                    var uwp_ida = new Windows.UI.Input.Inking.InkDrawingAttributes();
                    uwp_ida.Color = Windows.UI.Color.FromArgb(s.DrawingAttributes.Color.A,
                        s.DrawingAttributes.Color.R,
                        s.DrawingAttributes.Color.G,
                        s.DrawingAttributes.Color.B);
                    uwp_ida.Size = new Windows.Foundation.Size(s.DrawingAttributes.Width, s.DrawingAttributes.Height);   
                    if(s.DrawingAttributes.StylusTip == StylusTip.Ellipse)
                    {
                        uwp_ida.PenTip = Windows.UI.Input.Inking.PenTipShape.Circle;
                    }else if(s.DrawingAttributes.StylusTip == StylusTip.Rectangle)
                    {
                        uwp_ida.PenTip = Windows.UI.Input.Inking.PenTipShape.Rectangle;
                    }
                    uwp_ida.FitToCurve = s.DrawingAttributes.FitToCurve;
                    uwp_ida.PenTipTransform = new Matrix3x2((float)s.DrawingAttributes.StylusTipTransform.M11,
                        (float)s.DrawingAttributes.StylusTipTransform.M12,
                        (float)s.DrawingAttributes.StylusTipTransform.M21,
                        (float)s.DrawingAttributes.StylusTipTransform.M22,
                        (float)s.DrawingAttributes.StylusTipTransform.OffsetX,
                        (float)s.DrawingAttributes.StylusTipTransform.OffsetY);

                    uwp_s.DrawingAttributes = uwp_ida;
                    this._inkStrokeContainer.AddStroke(uwp_s);
                }

                var strokes = this._inkStrokeContainer.GetStrokes();
                var stroke = strokes[strokes.Count - 1];
                this.PasteFromClipboardTextBox.Text = stroke.GetInkPoints()[0].Position.X.ToString() + " "
                    + stroke.GetInkPoints()[0].Position.Y.ToString() + " "
                    + stroke.DrawingAttributes.Size.Width.ToString() + " "
                    + stroke.DrawingAttributes.Size.Height.ToString() + " last stroke info";
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void CopyToClipboardFromInkStrokeContainerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var strokes = this._inkStrokeContainer.GetStrokes();
            foreach (var stroke in strokes)
            {
                stroke.Selected = true; ;
            }
            this._inkStrokeContainer.CopySelectedToClipboard();


            this.MyProgressBar.IsIndeterminate = false;
        }
    }
}
